<?php
namespace Ak\Woo\User\Plugin;

use Ak\Form\View\Helper\FormControl;
use Ak\Gdpr\Entity\GdprConsent;
use Ak\Gdpr\Repository\GdprConsentRepository;
use Ak\Mailchimp\Gateway\MailingListGateway;
use Ak\Woo\User\Form\RegistrationForm;
use Ak\Wp\Plugin\PluginInterface;
use Exception;
use Laminas\Form\LabelAwareInterface;
use Laminas\ServiceManager\PluginManagerInterface;
use Laminas\View\Helper\HelperInterface;
use RuntimeException;
use WC_Coupon;
use WP_Error;

/**
 * Registracija korisnika.
 *
 * @package Ak\Woo\User\Plugin
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/autokreso/woo-user
 */
class Registration implements PluginInterface
{
    /** @var array */
    private $config;
    /** @var PluginManagerInterface */
    private $forms;
    /** @var RegistrationForm */
    private $form;
    /** @var FormControl */
    private $helper;
    /** @var GdprConsentRepository */
    private $gdprRepo;
    /** @var MailingListGateway */
    private $mailingLists;
    /** @var bool  */
    private $translationsLoaded = false;

    /**
     * Konstruktor.
     *
     * @param array $config
     * @param PluginManagerInterface $forms
     * @param HelperInterface $helper
     * @param GdprConsentRepository $gdprRepo
     * @param MailingListGateway $mailingLists
     */
    public function __construct(
        array $config,
        PluginManagerInterface $forms,
        HelperInterface $helper,
        GdprConsentRepository $gdprRepo,
        MailingListGateway $mailingLists
    ) {
        $this->config = $config;
        $this->forms = $forms;
        $this->helper = $helper;
        $this->gdprRepo = $gdprRepo;
        $this->mailingLists = $mailingLists;
    }

    /**
     * Dodaje akcije i filtere u WordPress.
     */
    public function attach(): void
    {
        add_action('woocommerce_after_checkout_validation', [$this, 'wc_after_checkout_validation'], 10, 2);
        add_filter('woocommerce_checkout_show_terms', [$this, 'wc_checkout_show_terms']);
        add_action('woocommerce_created_customer', [$this, 'wc_created_customer'], 10, 2);
        add_filter('woocommerce_process_registration_errors', [$this, 'wc_process_registration_errors']);
        add_action('woocommerce_register_form', [$this, 'wc_register_form']);
        add_action('woocommerce_review_order_before_submit', [$this, 'wc_register_form']);
        add_action('plugins_loaded', [$this, 'load_translations']);
        $this->load_translations();
    }

    public function load_translations()
    {
        if ($this->translationsLoaded) {
            return;
        }

        $locale = get_locale();

        if (preg_match('/^de_/', $locale)) {
            $locale = 'de';
        }

        $mofile = __DIR__ . '/../../i18n/' . $locale . '.mo';

        if (file_exists($mofile)) {
            load_textdomain('woo-user', $mofile);
        }
        $this->translationsLoaded = true;
    }

    /**
     * Validira registraciju prilikom naplate.
     *
     * @param $data
     * @param WP_Error $errors
     * @return WP_Error
     */
    public function wc_after_checkout_validation(/** @noinspection PhpUnusedParameterInspection */ $data, WP_Error $errors)
    {
        if ('POST' === $_SERVER['REQUEST_METHOD']) {
            try {
                $this->validateForm($errors);
            } catch (Exception $e) {
                // Ova validacija ne trpi exceptione.
            }
        }

        return $errors;
    }

    /**
     * Sakrivamo kvačicu za prihvaćanje uvjeta na stranici naplate.
     *
     * @return bool
     */
    public function wc_checkout_show_terms()
    {
        return false;
    }

    /**
     * Nakon registracije korisnika
     *
     * @param int $customer_id
     * @param array $customer_data
     */
    public function wc_created_customer($customer_id, $customer_data)
    {
        update_user_meta($customer_id, 'tos_accept', $_POST['tos_accept'] ?? '0');
        update_user_meta($customer_id, 'newsletter', $_POST['newsletter'] ?? '0');

        if ('1' === $_POST['tos_accept']) {
            $this->addGdprConsent($customer_data['user_email']);
        }

        if ('1' === $_POST['newsletter']) {
            $this->addNewsletterSubscription($customer_data['user_email']);
            $this->addNewsletterCoupon($customer_data['user_email']);
        }
    }

    /**
     * Renderira dodatna polja u obrazac.
     */
    public function wc_register_form()
    {
        $form = $this->getForm();

        $newsletter = $form->get('newsletter');
        echo $this->helper->render($newsletter);

        /** @var LabelAwareInterface $accept_tos */
        $acceptTos = $form->get('tos_accept');
        echo $this->helper->render($acceptTos);
    }

    /**
     * Validira registracijski obrazac.
     *
     * @param WP_Error $errors
     * @return WP_Error
     * @throws Exception
     */
    public function wc_process_registration_errors(WP_Error $errors)
    {
        if ('POST' === $_SERVER['REQUEST_METHOD']) {
            $this->validateForm($errors);
        }

        return $errors;
    }

    /**
     * Vraća registracijsku formu.
     *
     * @return RegistrationForm
     */
    private function getForm(): RegistrationForm
    {
        if (null === $this->form) {
            $this->form = $this->forms->get(RegistrationForm::class);
        }

        return $this->form;
    }

    /**
     * Dodaje GDPR privolu za datu email adresu.
     *
     * @param string $email
     */
    private function addGdprConsent(string $email): void
    {
        $consent = new GdprConsent();
        $consent->setEmail($email);
        $consent->setIp($_SERVER['REMOTE_ADDR']);

        $this->gdprRepo->insert($consent);
    }

    /**
     * Dodaje e-mail adresu u zadanu mailing listu.
     *
     * @param string $email
     */
    private function addNewsletterSubscription(string $email): void
    {
        $listId = $this->config['mailing_list_id'] ?? null;

        if ($listId) {
            $list = $this->mailingLists->getList($listId);

            if ($list) {
                $this->mailingLists->addListMember($list, $email);
            }
        }
    }

    /**
     * Dodaje korisnika u zadani kupon.
     *
     * @param string $email
     */
    private function addNewsletterCoupon(string $email): void
    {
        $couponId = $this->config['newsletter_coupon'] ?? null;

        if ($couponId) {
            $coupon = new WC_Coupon($couponId);
            $prev_value = $coupon->get_email_restrictions(null) ?: [];
            $new_value = $prev_value;

            if (!in_array($email, $prev_value)) {
                $new_value[] = $email;
                update_post_meta($coupon->get_id(), 'customer_email', $new_value, $prev_value);
            }
        }
    }

    /**
     * @param WP_Error $errors
     * @throws Exception
     */
    private function validateForm(WP_Error $errors)
    {
        $this->load_translations();
        $form = $this->getForm();
        $form->setData($_POST);

        if (!$form->isValid()) {
            $message = implode('', $form->getMessages('tos_accept'));
            $errors->add(1, $message);
            throw new Exception($message);
        }
    }
}
